@@ -185,6 +185,14 @@ def nComboBox(exname, name, default, popdown_strings):
             popdown_strings.index(default))
         c.set_active(popdown_strings.index(default))
     else:
+        try:
+            i = cfg.get_int("%s/%s" % (c.m_exname, c.m_name))
+        except ValueError:
+            i = 0
+            cfg.set_int("%s/%s" % (c.m_exname, c.m_name), 0)
+        if i >= len(popdown_strings):
+            i = 0
+            cfg.set_int("%s/%s" % (c.m_exname, c.m_name), 0)
         c.set_active(cfg.get_int("%s/%s" % (c.m_exname, c.m_name)))
     def f(combobox):
         cfg.set_int("%s/%s" % (exname, name), combobox.get_active())
