@@ -1,6 +1,6 @@
 # vim: set fileencoding=utf-8 :
 # GNU Solfege - free ear training software
-# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008  Tom Cato Amundsen
+# Copyright (C) 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2011  Tom Cato Amundsen
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -78,13 +78,11 @@ class SplashWin(gtk.Window):
 
 from solfege.configwindow import ConfigWindow
 from solfege.profilemanager import ChangeProfileDialog
-from solfege.mpd.musicdisplayer import MusicDisplayer
 from solfege import gu
 from solfege import cfg
+from solfege import mpd
 from solfege import stock
 from solfege import lessonfile
-from solfege import lessonfilegui
-from solfege import abstract
 from solfege import download_pyalsa
 from solfege import statistics
 
@@ -99,7 +97,7 @@ class MusicViewerWindow(gtk.Dialog):
     def __init__(self):
         gtk.Dialog.__init__(self)
         self.set_default_size(500, 300)
-        self.g_music_displayer = MusicDisplayer()
+        self.g_music_displayer = mpd.MusicDisplayer()
         self.vbox.pack_start(self.g_music_displayer)
         b = gu.bButton(self.action_area, _("Close"), solfege.win.close_musicviewer)
         b.grab_focus()
@@ -134,6 +132,7 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         self.g_config_window = None
         self.g_path_info_dlg = None
         self.g_musicviewer_window = None
+        self.m_history = []
         self.g_ui_manager = gtk.UIManager()
         self.m_action_groups = {
             'Exit': gtk.ActionGroup('Exit'),
@@ -279,6 +278,10 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
                 solfege.splash_win.show_progress(fn)
             if not frontpage.may_be_frontpage(fn):
                 continue
+            try:
+                title = lessonfile.infocache.frontpage.get(fn, 'title')
+            except TypeError:
+                continue
             cur_dir = os.path.split(fn)[0]
             if old_dir != cur_dir:
                 s += '<separator name="sep@%s"/>' % fn
@@ -393,8 +396,13 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         if not self.g_path_info_dlg:
             self.g_path_info_dlg = gtk.Dialog(_("_File locations").replace("_", ""), self,
                 buttons=(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
+            sc = gtk.ScrolledWindow()
+            sc.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_NEVER)
+            self.g_path_info_dlg.vbox.pack_start(sc, True, True)
+            #
             vbox = gu.hig_dlg_vbox()
-            self.g_path_info_dlg.vbox.pack_start(vbox)
+            sc.add_with_viewport(vbox)
+
             box1, box2 = gu.hig_category_vbox(_("_File locations").replace("_", ""))
             vbox.pack_start(box1)
             sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
@@ -415,8 +423,8 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
                 return True
             self.g_path_info_dlg.connect('response', f)
             self.g_path_info_dlg.connect('delete-event', f)
-        else:
-            self.g_path_info_dlg.show()
+            w, h = vbox.size_request()
+            sc.set_size_request(min(w + gu.hig_dlg_vbox.border_width * 2, gtk.gdk.screen_width()*0.9), h)
     def setup_pyalsa(self, widget):
         download_pyalsa.download()
     def show_about_window(self, widget):
@@ -515,6 +523,9 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
             print >> sys.stderr, "\n".join(textwrap.wrap("Translations are disabled because your locale settings are broken. This is not a bug in GNU Solfege, so don't report it. The README file distributed with the program has some more details."))
         except AttributeError:
             pass
+        for filename in lessonfile.infocache.frontpage.iter_old_format_files():
+            gu.dialog_ok(_("Cannot load front page file"), None,
+                _("The file «%s» is saved in an old file format. The file can be converted by editing and saving it with an older version of Solfege. Versions from 3.16.0 to 3.20.4 should do the job.") % filename)
     def activate_exercise(self, module, urlobj=None):
         self.show_view(module)
         # We need this test because not all exercises use a notebook.
@@ -540,27 +551,18 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
     def display_user_exercises(self, w):
         col = frontpage.Column()
         page = frontpage.Page(_('User exercises'), col)
-        curdir =None
-        for filename in lessonfile.infocache.iter_user_files():
+        curdir = None
+        linklist = None
+        for filename in lessonfile.infocache.iter_user_files(only_user_collection=True):
             dir, fn = os.path.split(filename)
             if dir != curdir:
                 curdir = dir
                 linklist = frontpage.LinkList(dir)
                 col.append(linklist)
             linklist.append(filename)
-        # Move ~/.solfege/exercises/user/lesson-files first in the list
-        # since we refer to this directory in the user manual
-        for idx, c in enumerate(col):
-            if c[0].startswith(os.path.join(filesystem.user_data(), u"exercises", u"user", u"lesson-files")):
-                col.insert(0, c)
-                del col[idx+1]
-                break
-        else:
-            linklist = frontpage.LinkList(os.path.join(filesystem.user_data(), u"exercises", u"user", u"lesson-files"))
-            col.insert(0, linklist)
-
         if os.path.isdir(filesystem.user_lessonfiles()):
             linklist = None
+            col.append(frontpage.Paragraph(_('You really should move the following directory to a directory below <span font_family="monospace">%s</span>. Future versions of GNU Solfege will not display files in the old location. The user manual have details on where to place the files.') % os.path.join(filesystem.user_data(), u'exercises')))
             # Added just to be nice with people not moving their files from
             # pre 3.15.3 location:
             for filename in os.listdir(filesystem.user_lessonfiles()):
@@ -591,15 +593,18 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         """
         self.set_title("GNU Solfege - tests")
         if not self.show_view('testspage'):
-            self.add_view(TestsView(), 'testspage')
+            p = TestsView()
+            p.connect('link-clicked', self.history_handler)
+            self.add_view(p, 'testspage')
         self.get_view().g_searchbox.show()
         if not data:
             data = solfege.app.m_frontpage_data
+        self.trim_history(self.get_view(), data)
         self.get_view().display_data(data, show_topics=show_topics)
     def on_search_all_exercises(self, widget=None):
         self.set_title("GNU Solfege")
         if not self.show_view('searchview'):
-            self.add_view(SearchView(u'Search the exercise titles of all lesson files found by the program, not just the active front page with sub pages.'), 'searchview')
+            self.add_view(SearchView(_('Search the exercise titles of all lesson files found by the program, not just the active front page with sub pages.')), 'searchview')
     def display_frontpage(self, data=None, show_topics=False):
         """
         Display the front page of the data  in solfege.app.m_frontpage_data
@@ -609,11 +614,23 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         else:
             self.set_title("GNU Solfege")
         if not self.show_view('frontpage'):
-            self.add_view(FrontPage(), 'frontpage')
+            p = FrontPage()
+            p.connect('link-clicked', self.history_handler)
+            self.add_view(p, 'frontpage')
         self.get_view().g_searchbox.show()
         if not data:
             data = solfege.app.m_frontpage_data
+        self.trim_history(self.get_view(), data)
         self.get_view().display_data(data, show_topics=show_topics)
+    def trim_history(self, new_viewer, new_page):
+        # First check if the page we want to display is in m_history.
+        # If so, we will trunkate history after it.
+        for i, (viewer, page) in enumerate(self.m_history):
+            if (new_viewer != viewer) or (new_page == page):
+                self.m_history = self.m_history[:i]
+                break
+    def history_handler(self, *w):
+        self.m_history.append(w)
     def initialise_exercise(self, teacher):
         """
         Create a Gui object for the exercise and add it to
@@ -621,13 +638,27 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         """
         assert teacher.m_exname not in self.box_dict
         self.get_view().hide()
-        m = __import__("solfege.exercises.%s" % teacher.m_exname, fromlist=("solfege.exercises.%s" % teacher.m_exname,), level=0)
+        m = solfege.app.import_module(teacher.m_exname)
         self.add_view(m.Gui(teacher), teacher.m_exname)
     def on_key_press_event(self, widget, event):
         try:
             view = self.get_view()
         except KeyError:
             return
+        if (event.type == gtk.gdk.KEY_PRESS
+            and event.state == gtk.gdk.MOD1_MASK # Alt key
+            and event.keyval in (gtk.keysyms.KP_Left, gtk.keysyms.Left)
+            and self.m_history
+            and not solfege.app.m_test_mode):
+            obj, page = self.m_history[-1]
+            self.trim_history(obj, page)
+            # Find the box_dict key for obj
+            for k, o in self.box_dict.items():
+                if o == obj:
+                    obj.display_data(page)
+                    self.show_view(k)
+                    break
+            return True
         view.on_key_press_event(widget, event)
     def open_profile_manager(self, widget=None):
         p = ChangeProfileDialog(solfege.app.m_options.profile)
@@ -643,6 +674,8 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
         solfege.app.m_options.profile = prof
         solfege.db.conn.commit()
         solfege.db.conn.close()
+        if prof == None:
+            prof = ''
         solfege.db = statistics.DB(None, profile=prof)
         cfg.set_string("app/last_profile", prof)
         self.display_frontpage()
