@@ -16,7 +16,7 @@ from solfege import dataparser
 from solfege import mpd
 from solfege.mpd import mpdutils
 from solfege import cfg
-from solfege.parsetree import FunctionCall, LookupException
+from solfege import parsetree as pt
 
 
 class MObjTest(TmpFileBase):
@@ -27,24 +27,24 @@ class MObjTest(TmpFileBase):
 
 class TestParser(TmpFileBase):
     parserclass = LessonfileCommon
-    def test_bad_musictype(self):
+    def test_wrong_argument_count(self):
         try:
             self.do_file('question {\nmusic = music("c e g", "chor") }')
-        except FunctionCall.WrongArgumentCount, e:
+        except pt.FunctionCall.WrongArgumentCount, e:
             self.assertEquals('(line 1): question {\n'
                               '(line 2): music = music("c e g", "chor") }\n'
                               '                        ^',
                               e.m_nonwrapped_text)
         else:
             self.fail("WrongArgumentCount not raised")
-    def test_bad_musictype_multiline(self):
+    def test_wrong_argument_count_multiline(self):
         try:
             self.do_file('\n'.join([
                 'question {',
                 'music = music("c e g",',
                 '   "chor")',
                 '}']))
-        except FunctionCall.WrongArgumentCount, e:
+        except pt.FunctionCall.WrongArgumentCount, e:
             self.assertEquals('(line 1): question {\n'
                               '(line 2): music = music("c e g",\n'
                               '                        ^',
@@ -55,7 +55,7 @@ class TestParser(TmpFileBase):
         try:
             self.do_file("a=1\n"
                          "b = func() c=3")
-        except LookupException, e:
+        except pt.LookupException, e:
             self.assertEquals("(line 1): a=1\n"
                               "(line 2): b = func() c=3\n"
                               "              ^",
@@ -66,7 +66,7 @@ class TestParser(TmpFileBase):
         try:
             self.do_file("a=1\n"
                          "b = func(3, 4) c=3")
-        except LookupException, e:
+        except pt.LookupException, e:
             #self.assertEquals(e.m_token[TOKEN_STRING], 'func')
             self.assertEquals("(line 1): a=1\n"
                               "(line 2): b = func(3, 4) c=3\n"
@@ -77,7 +77,7 @@ class TestParser(TmpFileBase):
     def test_exception_atom(self):
         try:
             self.do_file("a=b")
-        except LookupException, e:
+        except pt.LookupException, e:
             #self.assertEquals(e.m_token[TOKEN_STRING], "b")
             self.assertEquals("(line 1): a=b\n"
                               "            ^",
@@ -103,6 +103,7 @@ class TestParser(TmpFileBase):
         self.assertEquals(p.m_questions[0].s, "Dur")
         self.assertEquals(p.m_questions[0].s.cval, "Major")
     def test_predef(self):
+        self.p.m_module_predefs['treble'] = u'treble'
         p = self.do_file("header { clef = treble }")
         self.assert_(isinstance(p.header['clef'], unicode))
         self.assertEquals(p.header['clef'], 'treble')
